use crate::config;
use chrono::Local;
use tracing::Level;
use tracing_appender::non_blocking::WorkerGuard;
use tracing_subscriber::fmt;
use tracing_subscriber::fmt::format::Writer;
use tracing_subscriber::fmt::time::FormatTime;
use tracing_subscriber::fmt::writer::MakeWriterExt;
use tracing_subscriber::layer::SubscriberExt;
use tracing_subscriber::util::SubscriberInitExt;

// 格式化日志的输出时间格式
struct LocalTimer;

impl FormatTime for LocalTimer {
    fn format_time(&self, w: &mut Writer<'_>) -> std::fmt::Result {
        write!(w, "{}", Local::now().format("%Y-%m-%d %H:%M:%S"))
    }
}

/// 初始化日志
pub fn init_log() -> (WorkerGuard, WorkerGuard) {
    let config = config!();
    let logger_cfg = config.logger.clone();
    let (stdout_tracing_appender, std_guard) = tracing_appender::non_blocking(std::io::stdout());
    let (file_tracing_appender, file_guard) = tracing_appender::non_blocking(
        tracing_appender::rolling::daily(logger_cfg.dir, logger_cfg.prefix),
    );

    // 初始化并设置日志格式(定制和筛选日志)
    tracing_subscriber::registry()
        .with(
            fmt::layer()
                .with_writer(file_tracing_appender.with_max_level(package_level(&logger_cfg.level)))
                .with_ansi(false) // 关掉ansi的颜色输出功能
                // .json()
                .with_timer(LocalTimer),
        )
        .with(
            fmt::layer()
                .with_writer(
                    stdout_tracing_appender.with_max_level(package_level(&logger_cfg.level)),
                )
                .with_line_number(true) // 写入标准输出
                .with_ansi(true)
                .with_timer(LocalTimer)
                // .json()
                // .flatten_event(true)
            ,
        )
        .init(); // 初始化并将SubScriber设置为全局SubScriber

    (std_guard, file_guard)
}

fn package_level(level: &String) -> Level {
    match level.to_uppercase().as_str() {
        "TRACE" => Level::TRACE,
        "DEBUG" => Level::DEBUG,
        "INFO" => Level::INFO,
        "WARN" => Level::WARN,
        "ERROR" => Level::ERROR,
        _ => Level::INFO,
    }
}
