extern crate proc_macro2;
extern crate quote;
extern crate syn;

extern crate proc_macro;

use proc_macro::TokenStream; // 用于生成代码
use syn::{parse_macro_input, DeriveInput}; // 用于解析宏输入

mod responsable;
mod route;

/// `Responsable`的过程宏，将结构体实现`IntoResponse` trait
// #[proc_macro_derive(Responsable, attributes(status, headers))]
#[proc_macro_derive(Responsable)]
pub fn responsable(input: TokenStream) -> TokenStream {
    // 解析宏输入为`DeriveInput`
    let input = parse_macro_input!(input as DeriveInput);

    // 根据数据类型生成不同的代码
    responsable::gen_responsable(input)
}

/// route路由，
/// 
/// 第一个参数默认url
/// 
/// 第二个参数以methods = ["get", "post"] 等的形式设置路由支持的http method
#[proc_macro_attribute]
pub fn route(attr: TokenStream, item: TokenStream) -> TokenStream {
    route::gen_route(attr, item, "route")
}

/// get请求
/// 
/// 参数为url
#[proc_macro_attribute]
pub fn get(attr: TokenStream, item: TokenStream) -> TokenStream {
    route::gen_route(attr, item, "get")
}

/// post请求
/// 
/// 参数为url
#[proc_macro_attribute]
pub fn post(attr: TokenStream, item: TokenStream) -> TokenStream {
    route::gen_route(attr, item, "post")
}


/// put请求
/// 
/// 参数为url
#[proc_macro_attribute]
pub fn put(attr: TokenStream, item: TokenStream) -> TokenStream {
    route::gen_route(attr, item, "put")
}


/// patch请求
/// 
/// 参数为url
#[proc_macro_attribute]
pub fn patch(attr: TokenStream, item: TokenStream) -> TokenStream {
    route::gen_route(attr, item, "patch")
}

/// delete请求
/// 
/// 参数为url
#[proc_macro_attribute]
pub fn delete(attr: TokenStream, item: TokenStream) -> TokenStream {
    route::gen_route(attr, item, "delete")
}

/// option请求
/// 
/// 参数为url
#[proc_macro_attribute]
pub fn options(attr: TokenStream, item: TokenStream) -> TokenStream {
    route::gen_route(attr, item, "options")
}

/// head请求
/// 
/// 参数为url
#[proc_macro_attribute]
pub fn head(attr: TokenStream, item: TokenStream) -> TokenStream {
    route::gen_route(attr, item, "head")
}

/// trace请求
/// 
/// 参数为url
#[proc_macro_attribute]
pub fn trace(attr: TokenStream, item: TokenStream) -> TokenStream {
    route::gen_route(attr, item, "trace")
}